#include "lcd.h"
#include "i2c.h"
#include "type.h"

//GPIO Pins used
#define PCD8544_RESET (1<<9)
#define PCD8544_DC (1<<5)
#define SPI_SSEL (1<<7)

//Frame Interval
#define FRAME_INTERVAL 100

// PCD8544 Commandset
// ------------------
// General commands
#define PCD8544_POWERDOWN			0x04
#define PCD8544_ENTRYMODE			0x02
#define PCD8544_EXTENDEDINSTRUCTION	0x01
#define PCD8544_DISPLAYBLANK		0x00
#define PCD8544_DISPLAYNORMAL		0x04
#define PCD8544_DISPLAYALLON		0x01
#define PCD8544_DISPLAYINVERTED		0x05
// Normal instruction set
#define PCD8544_FUNCTIONSET			0x20
#define PCD8544_DISPLAYCONTROL		0x08
#define PCD8544_DISPLAYCHECK	0x01
#define PCD8544_SETYADDR			0x40
#define PCD8544_SETXADDR			0x80
// Extended instruction set
#define PCD8544_SETTEMP				0x04
#define PCD8544_SETBIAS				0x10
#define PCD8544_SETVOP				0x80
// Display presets
#define PCD8544_BIAS					0x03	// Range: 0-7 (0x00-0x07)
#define PCD8544_TEMP					0x02	// Range: 0-3 (0x00-0x03)
#define PCD8544_CONTRAST				0x46	// Range: 0-127 (0x00-0x7F)
//TIMER CODE BEGINS
#define PLOCK 0X00000400
#define PRESCALE 60000 //60000 PRECLOCK cycle to increment TC by 1

void PCD8544_write(unsigned char byte);

void PCD8544_init(void);

void PCD8544_gotoXY(unsigned char x, unsigned char y);

void PCD8544_cls(void);

void delayMS(unsigned int miliseconds);
void initClocks(void);
void initTimers(void);
void setupPLL0(void);
void feedSeq(void);
void connectPLL0(void);
void EEPROM_Inject(void);

//initializes SPI for transferring data to the controller
//pins 5 and 9 are used as GPIO pins and connects to the following
// 5 -> data/command |  9-> reset  
 void SPI_init()
 {  
 	//configure the pin connect block
 	PINSEL0 |= 0x00001100;   
 	
 	//GPIO pins for data/command and reset
 	IODIR0 |= 0x000002A0;	//Output configuration
    	
 	//clock divide of 8 for maximum SPI speed
 	S0SPCCR = 8;			
 	
 	// Control register configurations
 	S0SPCR = 0x0038;      
  }

//writes the byte parameter to location decided by X,Y address pointers
void PCD8544_write(unsigned char byte)
{
	IOCLR0 = SPI_SSEL; 				//SSEL low to enable device
	S0SPDR = byte;						//writes byte
	while(!(S0SPSR&(0x80)));	//checks SPIF bit for transfer completion
	IOSET0 = SPI_SSEL;				//SSEL high to stop transmission
}

//goto function to set address counters to X and Y.
void PCD8544_gotoXY(unsigned char x, unsigned char y)
{
	IOCLR0 = PCD8544_DC;		//command mode
	PCD8544_write(128+x);		//sets X address
	PCD8544_write(64+y);		//sets Y address
}

//clear screen functions to clear lcd and zero out  the RAM
void PCD8544_cls()
{
	int i;
	IOCLR0 = PCD8544_DC;			//command mode
	PCD8544_gotoXY(0,0);			
	IOSET0 = PCD8544_DC;			//data mode
	for(i=0; i<504;i++)
		PCD8544_write(0x00);
}

void initTimers(){
	T1CTCR= 0x0;
	T1PR = PRESCALE -1;//Increment T0TC at every 60000 clock cycles
	//Why subtract 1 = 60000 cycles at 60 MHz = 1 mS
	T1TCR = 0x02;

	//Timer 0 configured as Interrupt to update LCD
	T0CTCR = 0x0;       	//sets into timer mode
	T0PR = PRESCALE - 1; 	//increment T0TC every 60,000 clock cycles	 (hence every ms)
	T0MR0 = FRAME_INTERVAL;			
	T0MCR = 0x03;			//interrupt occurs and TC reset when match occurs with MR0
	T0TCR = 0x02;  		//counter reset
	T0TCR = 0x01;			//Counter enable
}

void delayMS( unsigned int ms){
	T1TCR = 0x02;//Counter Reset
	T1TCR = 0x01;//Counter Enable
	while (T1TC < ms);
	T1TCR=0x0;
}

void initClocks(){
	setupPLL0();
	feedSeq();
	connectPLL0();
	feedSeq();
	VPBDIV= 0x01; //PCLK = CCLK = 60 MHz
}

void setupPLL0(){
	//12 MHz crystal connected to LPC2148
	PLL0CON = 0x01;
	PLL0CFG = 0x24; 
}

void feedSeq(){
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
}

void connectPLL0(){
	while(!(PLL0STAT & PLOCK));
	PLL0CON=0x03;
}

void delayCALC(int time){//Delay by time units
	int i=0, j=0;
	for(;i<1000*time;i++)
	for(;j<10000;j++);
}

void delayIRQ(int count)    //you can use delay from timer to be precise and accurate instead of this
{
  int j=0,i=0;
  for(j=0;j<count;j++)
		for(i=0;i<60;i++);
}

//initializes the PCD5544 driver
void PCD8544_init()
{
	char read_buffer[1];
	read_buffer[0] = '1';
	initClocks(); //Init PCLK and CCLK at 60 MHz
	initTimers();
	init_lcd();
	lcd_clear();
	IOCLR0 = PCD8544_RESET;		//active low reset to initialize the device
	SPI_init(); 				//initialize the SPI on board
	IOSET0 = PCD8544_RESET;		//Reset disabled
	i2c_lpc_init(I2C_SPEED_100);		// Initialize I2C for memory code
	//Setting initial value in EEPROM
	//m24xx_write(EEPROM_24XX256, 0 , 0 ,(char *)read_buffer,sizeof(read_buffer));
	
	IOCLR0 = PCD8544_DC;		//D/C low to send commands
	//Sending commands
	PCD8544_write(PCD8544_FUNCTIONSET | PCD8544_EXTENDEDINSTRUCTION);	//To use extended instruction set  
	PCD8544_write(PCD8544_SETVOP | PCD8544_CONTRAST);	//sets operating voltage value Vop       
	PCD8544_write(PCD8544_SETTEMP | PCD8544_TEMP);		//sets temperature coefficient to 3       
	PCD8544_write(PCD8544_SETBIAS | PCD8544_BIAS);	// Bias value set to 3 (depends on multiplex rate) 
	PCD8544_write(PCD8544_FUNCTIONSET);	//to use basic instruction set    
	PCD8544_write(PCD8544_DISPLAYCONTROL | PCD8544_DISPLAYNORMAL);		//This is 8+4 which sets display configuration to normal mode
	//PCD8544_write(PCD8544_DISPLAYCONTROL | PCD8544_DISPLAYCHECK);	//Turn ON all segments
}
